@@ -4,13 +4,15 @@ use core::fmt;
 
 use generic_array::sequence::Concat;
 use generic_array::GenericArray;
-use rand_core::{OsRng, RngCore};
+use rand_core::{CryptoRng, RngCore};
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use typenum::{op, U32};
 
 use crate::curve::{CurvePoint, CurveScalar};
 use crate::hashing_ds::{hash_to_polynomial_arg, hash_to_shared_secret, kfrag_signature_message};
 use crate::keys::{PublicKey, SecretKey, Signature, Signer};
 use crate::params::Parameters;
+use crate::serde::{serde_deserialize, serde_serialize, Representation};
 use crate::traits::{
     fmt_public, ConstructionError, DeserializableFromArray, DeserializationError, HasTypeName,
     RepresentableAsArray, SerializableToArray,
@@ -24,9 +26,9 @@ type KeyFragIDSize = U32;
 pub(crate) struct KeyFragID(GenericArray<u8, KeyFragIDSize>);
 
 impl KeyFragID {
-    fn random() -> Self {
+    fn random(rng: &mut impl RngCore) -> Self {
         let mut bytes = GenericArray::<u8, KeyFragIDSize>::default();
-        OsRng.fill_bytes(&mut bytes);
+        rng.fill_bytes(&mut bytes);
         Self(bytes)
     }
 }
@@ -110,6 +112,7 @@ fn none_unless<T>(x: Option<T>, predicate: bool) -> Option<T> {
 
 impl KeyFragProof {
     fn from_base(
+        rng: &mut (impl CryptoRng + RngCore),
         base: &KeyFragBase,
         kfrag_id: &KeyFragID,
         kfrag_key: &CurveScalar,
@@ -121,9 +124,10 @@ impl KeyFragProof {
         let maybe_delegating_pk = Some(&base.delegating_pk);
         let maybe_receiving_pk = Some(&base.receiving_pk);
 
-        let signature_for_receiver = base.signer.sign(
+        let signature_for_receiver = base.signer.sign_with_rng(
+            rng,
             kfrag_signature_message(
-                &kfrag_id,
+                kfrag_id,
                 &commitment,
                 &base.precursor,
                 maybe_delegating_pk,
@@ -132,9 +136,10 @@ impl KeyFragProof {
             .as_ref(),
         );
 
-        let signature_for_proxy = base.signer.sign(
+        let signature_for_proxy = base.signer.sign_with_rng(
+            rng,
             kfrag_signature_message(
-                &kfrag_id,
+                kfrag_id,
                 &commitment,
                 &base.precursor,
                 none_unless(maybe_delegating_pk, sign_delegating_key),
@@ -198,6 +203,24 @@ impl DeserializableFromArray for KeyFrag {
     }
 }
 
+impl Serialize for KeyFrag {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serde_serialize(self, serializer, Representation::Base64)
+    }
+}
+
+impl<'de> Deserialize<'de> for KeyFrag {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        serde_deserialize(deserializer, Representation::Base64)
+    }
+}
+
 impl HasTypeName for KeyFrag {
     fn type_name() -> &'static str {
         "KeyFrag"
@@ -237,8 +260,13 @@ impl fmt::Display for KeyFragVerificationError {
 }
 
 impl KeyFrag {
-    fn from_base(base: &KeyFragBase, sign_delegating_key: bool, sign_receiving_key: bool) -> Self {
-        let kfrag_id = KeyFragID::random();
+    fn from_base(
+        rng: &mut (impl CryptoRng + RngCore),
+        base: &KeyFragBase,
+        sign_delegating_key: bool,
+        sign_receiving_key: bool,
+    ) -> Self {
+        let kfrag_id = KeyFragID::random(rng);
 
         // The index of the re-encryption key share (which in Shamir's Secret
         // Sharing corresponds to x in the tuple (x, f(x)), with f being the
@@ -256,7 +284,8 @@ impl KeyFrag {
         let rk = poly_eval(&base.coefficients, &share_index);
 
         let proof = KeyFragProof::from_base(
-            &base,
+            rng,
+            base,
             &kfrag_id,
             &rk,
             sign_delegating_key,
@@ -360,12 +389,13 @@ impl fmt::Display for VerifiedKeyFrag {
 
 impl VerifiedKeyFrag {
     pub(crate) fn from_base(
+        rng: &mut (impl CryptoRng + RngCore),
         base: &KeyFragBase,
         sign_delegating_key: bool,
         sign_receiving_key: bool,
     ) -> Self {
         Self {
-            kfrag: KeyFrag::from_base(base, sign_delegating_key, sign_receiving_key),
+            kfrag: KeyFrag::from_base(rng, base, sign_delegating_key, sign_receiving_key),
         }
     }
 
@@ -391,6 +421,7 @@ pub(crate) struct KeyFragBase {
 
 impl KeyFragBase {
     pub fn new(
+        rng: &mut (impl CryptoRng + RngCore),
         delegating_sk: &SecretKey,
         receiving_pk: &PublicKey,
         signer: &Signer,
@@ -406,7 +437,7 @@ impl KeyFragBase {
         let (d, precursor, dh_point) = loop {
             // The precursor point is used as an ephemeral public key in a DH key exchange,
             // and the resulting shared secret 'dh_point' is used to derive other secret values
-            let private_precursor = CurveScalar::random_nonzero();
+            let private_precursor = CurveScalar::random_nonzero(rng);
             let precursor = &g * &private_precursor;
 
             let dh_point = &receiving_pk_point * &private_precursor;
@@ -423,12 +454,12 @@ impl KeyFragBase {
 
         // Coefficients of the generating polynomial
         // `invert()` is guaranteed not to panic because `d` is nonzero.
-        let coefficient0 = &delegating_sk.to_secret_scalar() * &(d.invert().unwrap());
+        let coefficient0 = delegating_sk.to_secret_scalar().as_secret() * &(d.invert().unwrap());
 
         let mut coefficients = Vec::<CurveScalar>::with_capacity(threshold);
         coefficients.push(coefficient0);
         for _i in 1..threshold {
-            coefficients.push(CurveScalar::random_nonzero());
+            coefficients.push(CurveScalar::random_nonzero(rng));
         }
 
         Self {
@@ -457,7 +488,11 @@ mod tests {
 
     use alloc::boxed::Box;
 
+    use rand_core::OsRng;
+
     use super::{KeyFrag, KeyFragBase, KeyFragVerificationError, VerifiedKeyFrag};
+    use crate::serde::tests::{check_deserialization, check_serialization};
+    use crate::serde::Representation;
     use crate::{DeserializableFromArray, PublicKey, SecretKey, SerializableToArray, Signer};
 
     fn prepare_kfrags(
@@ -474,11 +509,11 @@ mod tests {
         let receiving_sk = SecretKey::random();
         let receiving_pk = receiving_sk.public_key();
 
-        let base = KeyFragBase::new(&delegating_sk, &receiving_pk, &signer, 2);
+        let base = KeyFragBase::new(&mut OsRng, &delegating_sk, &receiving_pk, &signer, 2);
         let vkfrags = [
-            VerifiedKeyFrag::from_base(&base, sign_delegating_key, sign_receiving_key),
-            VerifiedKeyFrag::from_base(&base, sign_delegating_key, sign_receiving_key),
-            VerifiedKeyFrag::from_base(&base, sign_delegating_key, sign_receiving_key),
+            VerifiedKeyFrag::from_base(&mut OsRng, &base, sign_delegating_key, sign_receiving_key),
+            VerifiedKeyFrag::from_base(&mut OsRng, &base, sign_delegating_key, sign_receiving_key),
+            VerifiedKeyFrag::from_base(&mut OsRng, &base, sign_delegating_key, sign_receiving_key),
         ];
 
         (delegating_pk, receiving_pk, verifying_pk, Box::new(vkfrags))
@@ -526,4 +561,16 @@ mod tests {
             }
         }
     }
+
+    #[test]
+    fn test_serde_serialization() {
+        let (_delegating_pk, _receiving_pk, _verifying_pk, verified_kfrags) =
+            prepare_kfrags(true, true);
+
+        let vkfrag = verified_kfrags[0].clone();
+        let kfrag = KeyFrag::from_array(&vkfrag.to_array()).unwrap();
+
+        check_serialization(&kfrag, Representation::Base64);
+        check_deserialization(&kfrag);
+    }
 }
